@@ -37,10 +37,13 @@
 #include "cairoint.h"
 
 #include "cairo-analysis-surface-private.h"
+#include "cairo-box-inline.h"
+#include "cairo-default-context-private.h"
 #include "cairo-error-private.h"
 #include "cairo-paginated-private.h"
-#include "cairo-recording-surface-private.h"
-#include "cairo-surface-subsurface-private.h"
+#include "cairo-recording-surface-inline.h"
+#include "cairo-surface-snapshot-inline.h"
+#include "cairo-surface-subsurface-inline.h"
 #include "cairo-region-private.h"
 
 typedef struct {
@@ -66,8 +69,8 @@ _cairo_analysis_surface_merge_status (cairo_int_status_t status_a,
 				      cairo_int_status_t status_b)
 {
     /* fatal errors should be checked and propagated at source */
-    assert (! _cairo_status_is_error (status_a));
-    assert (! _cairo_status_is_error (status_b));
+    assert (! _cairo_int_status_is_error (status_a));
+    assert (! _cairo_int_status_is_error (status_b));
 
     /* return the most important status */
     if (status_a == CAIRO_INT_STATUS_UNSUPPORTED ||
@@ -87,48 +90,126 @@ _cairo_analysis_surface_merge_status (cairo_int_status_t status_a,
 	return CAIRO_INT_STATUS_FLATTEN_TRANSPARENCY;
 
     /* at this point we have checked all the valid internal codes, so... */
-    assert (status_a == CAIRO_STATUS_SUCCESS &&
-	    status_b == CAIRO_STATUS_SUCCESS);
+    assert (status_a == CAIRO_INT_STATUS_SUCCESS &&
+	    status_b == CAIRO_INT_STATUS_SUCCESS);
 
+    return CAIRO_INT_STATUS_SUCCESS;
+}
+
+struct proxy {
+    cairo_surface_t base;
+    cairo_surface_t *target;
+};
+
+static cairo_status_t
+proxy_finish (void *abstract_surface)
+{
     return CAIRO_STATUS_SUCCESS;
 }
 
+static const cairo_surface_backend_t proxy_backend  = {
+    CAIRO_INTERNAL_SURFACE_TYPE_NULL,
+    proxy_finish,
+};
+
+static cairo_surface_t *
+attach_proxy (cairo_surface_t *source,
+	      cairo_surface_t *target)
+{
+    struct proxy *proxy;
+
+    proxy = malloc (sizeof (*proxy));
+    if (unlikely (proxy == NULL))
+	return _cairo_surface_create_in_error (CAIRO_STATUS_NO_MEMORY);
+
+    _cairo_surface_init (&proxy->base, &proxy_backend, NULL, target->content, target->is_vector);
+
+    proxy->target = target;
+    _cairo_surface_attach_snapshot (source, &proxy->base, NULL);
+
+    return &proxy->base;
+}
+
+static void
+detach_proxy (cairo_surface_t *proxy)
+{
+    cairo_surface_finish (proxy);
+    cairo_surface_destroy (proxy);
+}
+
 static cairo_int_status_t
 _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
-				    const cairo_pattern_t    *pattern)
+				    const cairo_pattern_t    *pattern,
+				    cairo_rectangle_int_t    *extents)
 {
     const cairo_surface_pattern_t *surface_pattern;
-    cairo_bool_t old_has_ctm;
-    cairo_matrix_t old_ctm, p2d;
-    cairo_status_t status;
-    cairo_surface_t *source;
+    cairo_analysis_surface_t *tmp;
+    cairo_surface_t *source, *proxy;
+    cairo_matrix_t p2d;
+    cairo_status_t status, analysis_status;
+    cairo_bool_t surface_is_unbounded;
+    cairo_bool_t unused;
 
     assert (pattern->type == CAIRO_PATTERN_TYPE_SURFACE);
     surface_pattern = (const cairo_surface_pattern_t *) pattern;
     assert (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING);
+    source = surface_pattern->surface;
 
-    old_ctm = surface->ctm;
-    old_has_ctm = surface->has_ctm;
+    proxy = _cairo_surface_has_snapshot (source, &proxy_backend);
+    if (proxy != NULL) {
+	/* nothing untoward found so far */
+	return CAIRO_STATUS_SUCCESS;
+    }
+
+    tmp = (cairo_analysis_surface_t *)
+	_cairo_analysis_surface_create (surface->target);
+    if (unlikely (tmp->base.status))
+	return tmp->base.status;
+    proxy = attach_proxy (source, &tmp->base);
 
     p2d = pattern->matrix;
     status = cairo_matrix_invert (&p2d);
     assert (status == CAIRO_STATUS_SUCCESS);
+    _cairo_analysis_surface_set_ctm (&tmp->base, &p2d);
 
-    cairo_matrix_multiply (&surface->ctm, &p2d, &surface->ctm);
-    surface->has_ctm = ! _cairo_matrix_is_identity (&surface->ctm);
-
-    source = surface_pattern->surface;
-    if (source->backend->type == CAIRO_SURFACE_TYPE_SUBSURFACE) {
-	cairo_surface_subsurface_t *sub = (cairo_surface_subsurface_t *) source;
-	source = sub->target;
+    source = _cairo_surface_get_source (source, NULL);
+    surface_is_unbounded = (pattern->extend == CAIRO_EXTEND_REPEAT
+				     || pattern->extend == CAIRO_EXTEND_REFLECT);
+    status = _cairo_recording_surface_replay_and_create_regions (source,
+								 &pattern->matrix,
+								 &tmp->base,
+								 surface_is_unbounded);
+    if (tmp->has_supported) {
+	surface->has_supported = TRUE;
+	unused = cairo_region_union (&surface->supported_region, &tmp->supported_region);
     }
 
-    status = _cairo_recording_surface_replay_and_create_regions (source, &surface->base);
+    if (tmp->has_unsupported) {
+	surface->has_unsupported = TRUE;
+	unused = cairo_region_union (&surface->fallback_region, &tmp->fallback_region);
+    }
 
-    surface->ctm = old_ctm;
-    surface->has_ctm = old_has_ctm;
+    analysis_status = tmp->has_unsupported ? CAIRO_INT_STATUS_IMAGE_FALLBACK : CAIRO_INT_STATUS_SUCCESS;
 
-    return status;
+    if (pattern->extend != CAIRO_EXTEND_NONE) {
+	_cairo_unbounded_rectangle_init (extents);
+    } else if (source->content & CAIRO_CONTENT_ALPHA) {
+	status = cairo_matrix_invert (&tmp->ctm);
+	_cairo_matrix_transform_bounding_box_fixed (&tmp->ctm,
+						    &tmp->page_bbox, NULL);
+	_cairo_box_round_to_rectangle (&tmp->page_bbox, extents);
+    } else {
+	/* black background fills entire extents */
+	_cairo_surface_get_extents (source, extents);
+    }
+
+    detach_proxy (proxy);
+    cairo_surface_destroy (&tmp->base);
+
+    if (unlikely (status))
+	return status;
+
+    return analysis_status;
 }
 
 static cairo_int_status_t
@@ -143,10 +224,11 @@ _add_operation (cairo_analysis_surface_t *surface,
 	/* Even though the operation is not visible we must be careful
 	 * to not allow unsupported operations to be replayed to the
 	 * backend during CAIRO_PAGINATED_MODE_RENDER */
-	if (backend_status == CAIRO_STATUS_SUCCESS ||
-	    backend_status == CAIRO_INT_STATUS_FLATTEN_TRANSPARENCY)
+	if (backend_status == CAIRO_INT_STATUS_SUCCESS ||
+	    backend_status == CAIRO_INT_STATUS_FLATTEN_TRANSPARENCY ||
+	    backend_status == CAIRO_INT_STATUS_NOTHING_TO_DO)
 	{
-	    return CAIRO_STATUS_SUCCESS;
+	    return CAIRO_INT_STATUS_SUCCESS;
 	}
 	else
 	{
@@ -162,6 +244,14 @@ _add_operation (cairo_analysis_surface_t *surface,
 	if (_cairo_matrix_is_integer_translation (&surface->ctm, &tx, &ty)) {
 	    rect->x += tx;
 	    rect->y += ty;
+
+	    tx = _cairo_fixed_from_int (tx);
+	    bbox.p1.x += tx;
+	    bbox.p2.x += tx;
+
+	    ty = _cairo_fixed_from_int (ty);
+	    bbox.p1.y += ty;
+	    bbox.p2.y += ty;
 	} else {
 	    _cairo_matrix_transform_bounding_box_fixed (&surface->ctm,
 							&bbox, NULL);
@@ -171,10 +261,11 @@ _add_operation (cairo_analysis_surface_t *surface,
 		 * careful to not allow unsupported operations to be
 		 * replayed to the backend during
 		 * CAIRO_PAGINATED_MODE_RENDER */
-		if (backend_status == CAIRO_STATUS_SUCCESS ||
-		    backend_status == CAIRO_INT_STATUS_FLATTEN_TRANSPARENCY)
+		if (backend_status == CAIRO_INT_STATUS_SUCCESS ||
+		    backend_status == CAIRO_INT_STATUS_FLATTEN_TRANSPARENCY ||
+		    backend_status == CAIRO_INT_STATUS_NOTHING_TO_DO)
 		{
-		    return CAIRO_STATUS_SUCCESS;
+		    return CAIRO_INT_STATUS_SUCCESS;
 		}
 		else
 		{
@@ -189,16 +280,8 @@ _add_operation (cairo_analysis_surface_t *surface,
     if (surface->first_op) {
 	surface->first_op = FALSE;
 	surface->page_bbox = bbox;
-    } else {
-	if (bbox.p1.x < surface->page_bbox.p1.x)
-	    surface->page_bbox.p1.x = bbox.p1.x;
-	if (bbox.p1.y < surface->page_bbox.p1.y)
-	    surface->page_bbox.p1.y = bbox.p1.y;
-	if (bbox.p2.x > surface->page_bbox.p2.x)
-	    surface->page_bbox.p2.x = bbox.p2.x;
-	if (bbox.p2.y > surface->page_bbox.p2.y)
-	    surface->page_bbox.p2.y = bbox.p2.y;
-    }
+    } else
+	_cairo_box_add_box(&surface->page_bbox, &bbox);
 
     /* If the operation is completely enclosed within the fallback
      * region there is no benefit in emitting a native operation as
@@ -216,10 +299,10 @@ _add_operation (cairo_analysis_surface_t *surface,
 	 * transparency into the white background.
 	 */
 	if (cairo_region_contains_rectangle (&surface->supported_region, rect) == CAIRO_REGION_OVERLAP_OUT)
-	    backend_status = CAIRO_STATUS_SUCCESS;
+	    backend_status = CAIRO_INT_STATUS_SUCCESS;
     }
 
-    if (backend_status == CAIRO_STATUS_SUCCESS) {
+    if (backend_status == CAIRO_INT_STATUS_SUCCESS) {
 	/* Add the operation to the supported region. Operations in
 	 * this region will be emitted as native operations.
 	 */
@@ -240,7 +323,7 @@ _add_operation (cairo_analysis_surface_t *surface,
      * invoke the cairo-surface-fallback path then return
      * CAIRO_STATUS_SUCCESS.
      */
-    if (status == CAIRO_STATUS_SUCCESS)
+    if (status == CAIRO_INT_STATUS_SUCCESS)
 	return CAIRO_INT_STATUS_IMAGE_FALLBACK;
     else
 	return status;
@@ -269,24 +352,17 @@ _cairo_analysis_surface_get_extents (void			*abstract_surface,
 }
 
 static void
-_rectangle_intersect_clip (cairo_rectangle_int_t *extents, cairo_clip_t *clip)
+_rectangle_intersect_clip (cairo_rectangle_int_t *extents, const cairo_clip_t *clip)
 {
-    const cairo_rectangle_int_t *clip_extents;
-    cairo_bool_t is_empty;
-
-    clip_extents = NULL;
     if (clip != NULL)
-	clip_extents = _cairo_clip_get_extents (clip);
-
-    if (clip_extents != NULL)
-	is_empty = _cairo_rectangle_intersect (extents, clip_extents);
+	_cairo_rectangle_intersect (extents, _cairo_clip_get_extents (clip));
 }
 
 static void
 _cairo_analysis_surface_operation_extents (cairo_analysis_surface_t *surface,
 					   cairo_operator_t op,
 					   const cairo_pattern_t *source,
-					   cairo_clip_t *clip,
+					   const cairo_clip_t *clip,
 					   cairo_rectangle_int_t *extents)
 {
     cairo_bool_t is_empty;
@@ -296,8 +372,8 @@ _cairo_analysis_surface_operation_extents (cairo_analysis_surface_t *surface,
     if (_cairo_operator_bounded_by_source (op)) {
 	cairo_rectangle_int_t source_extents;
 
-	_cairo_pattern_get_extents (source, &source_extents);
-	is_empty = _cairo_rectangle_intersect (extents, &source_extents);
+	_cairo_pattern_get_extents (source, &source_extents, surface->target->is_vector);
+	_cairo_rectangle_intersect (extents, &source_extents);
     }
 
     _rectangle_intersect_clip (extents, clip);
@@ -307,10 +383,10 @@ static cairo_int_status_t
 _cairo_analysis_surface_paint (void			*abstract_surface,
 			       cairo_operator_t		op,
 			       const cairo_pattern_t	*source,
-			       cairo_clip_t		*clip)
+			       const cairo_clip_t	*clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
-    cairo_status_t	     backend_status;
+    cairo_int_status_t	     backend_status;
     cairo_rectangle_int_t  extents;
 
     if (surface->target->backend->paint == NULL) {
@@ -319,16 +395,18 @@ _cairo_analysis_surface_paint (void			*abstract_surface,
 	backend_status =
 	    surface->target->backend->paint (surface->target,
 					     op, source, clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
 
-    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN)
-	backend_status = _analyze_recording_surface_pattern (surface, source);
-
     _cairo_analysis_surface_operation_extents (surface,
 					       op, source, clip,
 					       &extents);
+    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	cairo_rectangle_int_t rec_extents;
+	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_cairo_rectangle_intersect (&extents, &rec_extents);
+    }
 
     return _add_operation (surface, &extents, backend_status);
 }
@@ -338,12 +416,11 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
 			      cairo_operator_t		 op,
 			      const cairo_pattern_t	*source,
 			      const cairo_pattern_t	*mask,
-			      cairo_clip_t		*clip)
+			      const cairo_clip_t	*clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
     cairo_int_status_t	      backend_status;
     cairo_rectangle_int_t   extents;
-    cairo_bool_t is_empty;
 
     if (surface->target->backend->mask == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
@@ -351,31 +428,41 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
 	backend_status =
 	    surface->target->backend->mask (surface->target,
 					    op, source, mask, clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
 
+    _cairo_analysis_surface_operation_extents (surface,
+					       op, source, clip,
+					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_int_status_t backend_source_status = CAIRO_STATUS_SUCCESS;
 	cairo_int_status_t backend_mask_status = CAIRO_STATUS_SUCCESS;
+	cairo_rectangle_int_t rec_extents;
 
 	if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
-	    const cairo_surface_pattern_t *surface_pattern = (const cairo_surface_pattern_t *) source;
-	    if (_cairo_surface_is_recording (surface_pattern->surface)) {
+	    cairo_surface_t *src_surface = ((cairo_surface_pattern_t *)source)->surface;
+	    src_surface = _cairo_surface_get_source (src_surface, NULL);
+	    if (_cairo_surface_is_recording (src_surface)) {
 		backend_source_status =
-		    _analyze_recording_surface_pattern (surface, source);
-		if (_cairo_status_is_error (backend_source_status))
+		    _analyze_recording_surface_pattern (surface, source, &rec_extents);
+		if (_cairo_int_status_is_error (backend_source_status))
 		    return backend_source_status;
+
+		_cairo_rectangle_intersect (&extents, &rec_extents);
 	    }
 	}
 
 	if (mask->type == CAIRO_PATTERN_TYPE_SURFACE) {
-	    cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) mask;
-	    if (_cairo_surface_is_recording (surface_pattern->surface)) {
+	    cairo_surface_t *mask_surface = ((cairo_surface_pattern_t *)mask)->surface;
+	    mask_surface = _cairo_surface_get_source (mask_surface, NULL);
+	    if (_cairo_surface_is_recording (mask_surface)) {
 		backend_mask_status =
-		    _analyze_recording_surface_pattern (surface, mask);
-		if (_cairo_status_is_error (backend_mask_status))
+		    _analyze_recording_surface_pattern (surface, mask, &rec_extents);
+		if (_cairo_int_status_is_error (backend_mask_status))
 		    return backend_mask_status;
+
+		_cairo_rectangle_intersect (&extents, &rec_extents);
 	    }
 	}
 
@@ -384,37 +471,31 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
 						  backend_mask_status);
     }
 
-    _cairo_analysis_surface_operation_extents (surface,
-					       op, source, clip,
-					       &extents);
-
     if (_cairo_operator_bounded_by_mask (op)) {
 	cairo_rectangle_int_t mask_extents;
 
-	_cairo_pattern_get_extents (mask, &mask_extents);
-	is_empty = _cairo_rectangle_intersect (&extents, &mask_extents);
-
+	_cairo_pattern_get_extents (mask, &mask_extents, surface->target->is_vector);
+	_cairo_rectangle_intersect (&extents, &mask_extents);
     }
 
     return _add_operation (surface, &extents, backend_status);
 }
 
 static cairo_int_status_t
-_cairo_analysis_surface_stroke (void			*abstract_surface,
-				cairo_operator_t	 op,
-				const cairo_pattern_t	*source,
-				cairo_path_fixed_t	*path,
-				const cairo_stroke_style_t	*style,
-				const cairo_matrix_t		*ctm,
-				const cairo_matrix_t		*ctm_inverse,
-				double			 tolerance,
-				cairo_antialias_t	 antialias,
-				cairo_clip_t		*clip)
+_cairo_analysis_surface_stroke (void			   *abstract_surface,
+				cairo_operator_t	    op,
+				const cairo_pattern_t	   *source,
+				const cairo_path_fixed_t   *path,
+				const cairo_stroke_style_t *style,
+				const cairo_matrix_t	   *ctm,
+				const cairo_matrix_t	   *ctm_inverse,
+				double			    tolerance,
+				cairo_antialias_t	    antialias,
+				const cairo_clip_t	   *clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
-    cairo_status_t	     backend_status;
+    cairo_int_status_t	     backend_status;
     cairo_rectangle_int_t    extents;
-    cairo_bool_t             is_empty;
 
     if (surface->target->backend->stroke == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
@@ -425,20 +506,22 @@ _cairo_analysis_surface_stroke (void			*abstract_surface,
 					      ctm, ctm_inverse,
 					      tolerance, antialias,
 					      clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
 
-    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN)
-	backend_status = _analyze_recording_surface_pattern (surface, source);
-
     _cairo_analysis_surface_operation_extents (surface,
 					       op, source, clip,
 					       &extents);
+    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	cairo_rectangle_int_t rec_extents;
+	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_cairo_rectangle_intersect (&extents, &rec_extents);
+    }
 
     if (_cairo_operator_bounded_by_mask (op)) {
 	cairo_rectangle_int_t mask_extents;
-	cairo_status_t status;
+	cairo_int_status_t status;
 
 	status = _cairo_path_fixed_stroke_extents (path, style,
 						   ctm, ctm_inverse,
@@ -447,7 +530,7 @@ _cairo_analysis_surface_stroke (void			*abstract_surface,
 	if (unlikely (status))
 	    return status;
 
-	is_empty = _cairo_rectangle_intersect (&extents, &mask_extents);
+	_cairo_rectangle_intersect (&extents, &mask_extents);
     }
 
     return _add_operation (surface, &extents, backend_status);
@@ -457,16 +540,15 @@ static cairo_int_status_t
 _cairo_analysis_surface_fill (void			*abstract_surface,
 			      cairo_operator_t		 op,
 			      const cairo_pattern_t	*source,
-			      cairo_path_fixed_t	*path,
+			      const cairo_path_fixed_t	*path,
 			      cairo_fill_rule_t		 fill_rule,
 			      double			 tolerance,
 			      cairo_antialias_t		 antialias,
-			      cairo_clip_t		*clip)
+			      const cairo_clip_t	*clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
-    cairo_status_t	     backend_status;
+    cairo_int_status_t	     backend_status;
     cairo_rectangle_int_t    extents;
-    cairo_bool_t             is_empty;
 
     if (surface->target->backend->fill == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
@@ -476,16 +558,18 @@ _cairo_analysis_surface_fill (void			*abstract_surface,
 					    source, path, fill_rule,
 					    tolerance, antialias,
 					    clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
 
-    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN)
-	backend_status = _analyze_recording_surface_pattern (surface, source);
-
     _cairo_analysis_surface_operation_extents (surface,
 					       op, source, clip,
 					       &extents);
+    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	cairo_rectangle_int_t rec_extents;
+	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_cairo_rectangle_intersect (&extents, &rec_extents);
+    }
 
     if (_cairo_operator_bounded_by_mask (op)) {
 	cairo_rectangle_int_t mask_extents;
@@ -493,7 +577,7 @@ _cairo_analysis_surface_fill (void			*abstract_surface,
 	_cairo_path_fixed_fill_extents (path, fill_rule, tolerance,
 					&mask_extents);
 
-	is_empty = _cairo_rectangle_intersect (&extents, &mask_extents);
+	_cairo_rectangle_intersect (&extents, &mask_extents);
     }
 
     return _add_operation (surface, &extents, backend_status);
@@ -506,13 +590,11 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 				     cairo_glyph_t	  *glyphs,
 				     int		   num_glyphs,
 				     cairo_scaled_font_t  *scaled_font,
-				     cairo_clip_t         *clip,
-				     int                  *remaining_glyphs)
+				     const cairo_clip_t         *clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
-    cairo_status_t	     status, backend_status;
+    cairo_int_status_t	     status, backend_status;
     cairo_rectangle_int_t    extents, glyph_extents;
-    cairo_bool_t             is_empty;
 
     /* Adapted from _cairo_surface_show_glyphs */
     if (surface->target->backend->show_glyphs != NULL) {
@@ -521,9 +603,8 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 						   source,
 						   glyphs, num_glyphs,
 						   scaled_font,
-						   clip,
-						   remaining_glyphs);
-	if (_cairo_status_is_error (backend_status))
+						   clip);
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
     else if (surface->target->backend->show_text_glyphs != NULL)
@@ -537,7 +618,7 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 							FALSE,
 							scaled_font,
 							clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
     else
@@ -545,12 +626,14 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
     }
 
-    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN)
-	backend_status = _analyze_recording_surface_pattern (surface, source);
-
     _cairo_analysis_surface_operation_extents (surface,
 					       op, source, clip,
 					       &extents);
+    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	cairo_rectangle_int_t rec_extents;
+	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_cairo_rectangle_intersect (&extents, &rec_extents);
+    }
 
     if (_cairo_operator_bounded_by_mask (op)) {
 	status = _cairo_scaled_font_glyph_device_extents (scaled_font,
@@ -561,7 +644,7 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 	if (unlikely (status))
 	    return status;
 
-	is_empty = _cairo_rectangle_intersect (&extents, &glyph_extents);
+	_cairo_rectangle_intersect (&extents, &glyph_extents);
     }
 
     return _add_operation (surface, &extents, backend_status);
@@ -587,12 +670,11 @@ _cairo_analysis_surface_show_text_glyphs (void			    *abstract_surface,
 					  int			     num_clusters,
 					  cairo_text_cluster_flags_t cluster_flags,
 					  cairo_scaled_font_t	    *scaled_font,
-					  cairo_clip_t		    *clip)
+					  const cairo_clip_t		    *clip)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
-    cairo_status_t	     status, backend_status;
+    cairo_int_status_t	     status, backend_status;
     cairo_rectangle_int_t    extents, glyph_extents;
-    cairo_bool_t             is_empty;
 
     /* Adapted from _cairo_surface_show_glyphs */
     backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
@@ -606,35 +688,30 @@ _cairo_analysis_surface_show_text_glyphs (void			    *abstract_surface,
 							cluster_flags,
 							scaled_font,
 							clip);
-	if (_cairo_status_is_error (backend_status))
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
     }
     if (backend_status == CAIRO_INT_STATUS_UNSUPPORTED &&
 	surface->target->backend->show_glyphs != NULL)
     {
-	int remaining_glyphs = num_glyphs;
 	backend_status =
 	    surface->target->backend->show_glyphs (surface->target, op,
 						   source,
 						   glyphs, num_glyphs,
 						   scaled_font,
-						   clip,
-						   &remaining_glyphs);
-	if (_cairo_status_is_error (backend_status))
+						   clip);
+	if (_cairo_int_status_is_error (backend_status))
 	    return backend_status;
-
-	glyphs += num_glyphs - remaining_glyphs;
-	num_glyphs = remaining_glyphs;
-	if (remaining_glyphs == 0)
-	    backend_status = CAIRO_STATUS_SUCCESS;
     }
 
-    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN)
-	backend_status = _analyze_recording_surface_pattern (surface, source);
-
     _cairo_analysis_surface_operation_extents (surface,
 					       op, source, clip,
 					       &extents);
+    if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	cairo_rectangle_int_t rec_extents;
+	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_cairo_rectangle_intersect (&extents, &rec_extents);
+    }
 
     if (_cairo_operator_bounded_by_mask (op)) {
 	status = _cairo_scaled_font_glyph_device_extents (scaled_font,
@@ -645,47 +722,80 @@ _cairo_analysis_surface_show_text_glyphs (void			    *abstract_surface,
 	if (unlikely (status))
 	    return status;
 
-	is_empty = _cairo_rectangle_intersect (&extents, &glyph_extents);
+	_cairo_rectangle_intersect (&extents, &glyph_extents);
     }
 
     return _add_operation (surface, &extents, backend_status);
 }
 
+static cairo_int_status_t
+_cairo_analysis_surface_tag (void	                *abstract_surface,
+			     cairo_bool_t                begin,
+			     const char                 *tag_name,
+			     const char                 *attributes,
+			     const cairo_pattern_t	*source,
+			     const cairo_stroke_style_t	*stroke_style,
+			     const cairo_matrix_t	*ctm,
+			     const cairo_matrix_t	*ctm_inverse,
+			     const cairo_clip_t	        *clip)
+{
+    cairo_analysis_surface_t *surface = abstract_surface;
+    cairo_int_status_t	     backend_status;
+
+    backend_status = CAIRO_INT_STATUS_SUCCESS;
+    if (surface->target->backend->tag != NULL) {
+	backend_status =
+	    surface->target->backend->tag (surface->target,
+					   begin,
+					   tag_name,
+					   attributes,
+					   source,
+					   stroke_style,
+					   ctm,
+					   ctm_inverse,
+					   clip);
+	if (_cairo_int_status_is_error (backend_status))
+	    return backend_status;
+    }
+
+    return backend_status;
+}
+
 static const cairo_surface_backend_t cairo_analysis_surface_backend = {
     CAIRO_INTERNAL_SURFACE_TYPE_ANALYSIS,
-    NULL, /* create_similar */
+
     _cairo_analysis_surface_finish,
+    NULL,
+
+    NULL, /* create_similar */
+    NULL, /* create_similar_image */
+    NULL, /* map_to_image */
+    NULL, /* unmap */
+
+    NULL, /* source */
     NULL, /* acquire_source_image */
     NULL, /* release_source_image */
-    NULL, /* acquire_dest_image */
-    NULL, /* release_dest_image */
-    NULL, /* clone_similar */
-    NULL, /* composite */
-    NULL, /* fill_rectangles */
-    NULL, /* composite_trapezoids */
-    NULL, /* create_span_renderer */
-    NULL, /* check_span_renderer */
+    NULL, /* snapshot */
+
     NULL, /* copy_page */
     NULL, /* show_page */
+
     _cairo_analysis_surface_get_extents,
-    NULL, /* old_show_glyphs */
     NULL, /* get_font_options */
+
     NULL, /* flush */
     NULL, /* mark_dirty_rectangle */
-    NULL, /* scaled_font_fini */
-    NULL, /* scaled_glyph_fini */
+
     _cairo_analysis_surface_paint,
     _cairo_analysis_surface_mask,
     _cairo_analysis_surface_stroke,
     _cairo_analysis_surface_fill,
-    _cairo_analysis_surface_show_glyphs,
-    NULL, /* snapshot */
-    NULL, /* is_similar */
     NULL, /* fill_stroke */
-    NULL, /* create_solid_pattern_surface */
-    NULL, /* can_repaint_solid_pattern_surface */
+    _cairo_analysis_surface_show_glyphs,
     _cairo_analysis_surface_has_show_text_glyphs,
-    _cairo_analysis_surface_show_text_glyphs
+    _cairo_analysis_surface_show_text_glyphs,
+    NULL, /* get_supported_mime_types */
+    _cairo_analysis_surface_tag
 };
 
 cairo_surface_t *
@@ -707,7 +817,8 @@ _cairo_analysis_surface_create (cairo_surface_t		*target)
     _cairo_surface_init (&surface->base,
 			 &cairo_analysis_surface_backend,
 			 NULL, /* device */
-			 CAIRO_CONTENT_COLOR_ALPHA);
+			 CAIRO_CONTENT_COLOR_ALPHA,
+			 target->is_vector);
 
     cairo_matrix_init_identity (&surface->ctm);
     surface->has_ctm = FALSE;
@@ -807,36 +918,36 @@ typedef cairo_int_status_t
 (*_paint_func)			(void			*surface,
 			         cairo_operator_t	 op,
 				 const cairo_pattern_t	*source,
-				 cairo_clip_t		*clip);
+				 const cairo_clip_t		*clip);
 
 typedef cairo_int_status_t
 (*_mask_func)			(void			*surface,
 			         cairo_operator_t	 op,
 				 const cairo_pattern_t	*source,
 				 const cairo_pattern_t	*mask,
-				 cairo_clip_t		*clip);
+				 const cairo_clip_t		*clip);
 
 typedef cairo_int_status_t
 (*_stroke_func)			(void			*surface,
 			         cairo_operator_t	 op,
 				 const cairo_pattern_t	*source,
-				 cairo_path_fixed_t	*path,
+				 const cairo_path_fixed_t	*path,
 				 const cairo_stroke_style_t	*style,
 				 const cairo_matrix_t		*ctm,
 				 const cairo_matrix_t		*ctm_inverse,
 				 double			 tolerance,
 				 cairo_antialias_t	 antialias,
-				 cairo_clip_t		*clip);
+				 const cairo_clip_t		*clip);
 
 typedef cairo_int_status_t
 (*_fill_func)			(void			*surface,
 			         cairo_operator_t	 op,
 				 const cairo_pattern_t	*source,
-				 cairo_path_fixed_t	*path,
+				 const cairo_path_fixed_t	*path,
 				 cairo_fill_rule_t	 fill_rule,
 				 double			 tolerance,
 				 cairo_antialias_t	 antialias,
-				 cairo_clip_t		*clip);
+				 const cairo_clip_t		*clip);
 
 typedef cairo_int_status_t
 (*_show_glyphs_func)		(void			*surface,
@@ -845,43 +956,39 @@ typedef cairo_int_status_t
 				 cairo_glyph_t		*glyphs,
 				 int			 num_glyphs,
 				 cairo_scaled_font_t	*scaled_font,
-				 cairo_clip_t		*clip,
-				 int			*remaining_glyphs);
+				 const cairo_clip_t		*clip);
 
 static const cairo_surface_backend_t cairo_null_surface_backend = {
     CAIRO_INTERNAL_SURFACE_TYPE_NULL,
+    NULL, /* finish */
+
+    NULL, /* only accessed through the surface functions */
 
     NULL, /* create_similar */
-    NULL, /* finish */
+    NULL, /* create similar image */
+    NULL, /* map to image */
+    NULL, /* unmap image*/
+
+    NULL, /* source */
     NULL, /* acquire_source_image */
     NULL, /* release_source_image */
-    NULL, /* acquire_dest_image */
-    NULL, /* release_dest_image */
-    NULL, /* clone_similar */
-    NULL, /* composite */
-    NULL, /* fill_rectangles */
-    NULL, /* composite_trapezoids */
-    NULL, /* create_span_renderer */
-    NULL, /* check_span_renderer */
+    NULL, /* snapshot */
+
     NULL, /* copy_page */
     NULL, /* show_page */
+
     NULL, /* get_extents */
-    NULL, /* old_show_glyphs */
     NULL, /* get_font_options */
+
     NULL, /* flush */
     NULL, /* mark_dirty_rectangle */
-    NULL, /* scaled_font_fini */
-    NULL, /* scaled_glyph_fini */
+
     (_paint_func) _return_success,	    /* paint */
     (_mask_func) _return_success,	    /* mask */
     (_stroke_func) _return_success,	    /* stroke */
     (_fill_func) _return_success,	    /* fill */
-    (_show_glyphs_func) _return_success,    /* show_glyphs */
-    NULL, /* snapshot */
-    NULL, /* is_similar */
     NULL, /* fill_stroke */
-    NULL, /* create_solid_pattern_surface */
-    NULL, /* can_repaint_solid_pattern_surface */
+    (_show_glyphs_func) _return_success,    /* show_glyphs */
     NULL, /* has_show_text_glyphs */
     NULL  /* show_text_glyphs */
 };
@@ -899,7 +1006,8 @@ _cairo_null_surface_create (cairo_content_t content)
     _cairo_surface_init (surface,
 			 &cairo_null_surface_backend,
 			 NULL, /* device */
-			 content);
+			 content,
+			 TRUE); /* is_vector */
 
     return surface;
 }
