@@ -456,7 +456,7 @@ edges_compare_x_for_y (const cairo_edge_t *a,
        HAVE_BX      = 0x2,
        HAVE_BOTH    = HAVE_AX | HAVE_BX
     } have_ax_bx = HAVE_BOTH;
-    int32_t ax, bx;
+    int32_t ax = 0, bx = 0;
 
     /* XXX given we have x and dx? */
 
@@ -2147,8 +2147,8 @@ _cairo_botor_scan_converter_add_edge (void		*converter,
     return botor_add_edge (self, &edge);
 }
 
-static cairo_status_t
-_cairo_botor_scan_converter_add_polygon (void		*converter,
+cairo_status_t
+_cairo_botor_scan_converter_add_polygon (cairo_botor_scan_converter_t *converter,
 					 const cairo_polygon_t *polygon)
 {
     cairo_botor_scan_converter_t *self = converter;
